<?php
namespace ElementorTMusers;

class TMusersLoadElementor {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function widget_styles() {
		wp_register_style( 'tmusers-components', plugins_url( '/css/components.css', __FILE__ ), false, '1.0.2' );
        wp_register_style( 'tmusers-h-card', plugins_url( '/css/h-card.css', __FILE__ ), false, '1.0' );
		wp_register_style( 'tmusers-v-card', plugins_url( '/css/v-card.css', __FILE__ ), false, '1.0' );
		wp_register_style( 'tmusers-t-card', plugins_url( '/css/t-card.css', __FILE__ ), false, '1.0' );
		wp_register_style( 'tmusers-table', plugins_url( '/css/table.css', __FILE__ ), false, '1.0' );
	}

	public function widget_scripts() {
		wp_register_script( 'tmusers-h-card', plugins_url( '/js/h-card.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'tmusers-v-card', plugins_url( '/js/v-card.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'tmusers-t-card', plugins_url( '/js/t-card.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'tmusers-data-table', plugins_url( '/js/data-table.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'tmusers-table', plugins_url( '/js/table.js', __FILE__ ), [ 'jquery' ], false, true );
		$tmusers_table_param = array(
			"lengthMenu" => esc_html__( 'Display _MENU_ records per page', 'tmusers' ),
			"zeroRecords" => esc_html__( 'Nothing found - sorry', 'tmusers' ),
			"info" => esc_html__( 'Showing page _PAGE_ of _PAGES_', 'tmusers' ),
			"infoEmpty" => esc_html__( 'No records available', 'tmusers' ),
			"infoFiltered" => esc_html__( '(filtered from _MAX_ total records)', 'tmusers' ),
			"searchPlaceholder" => esc_html__( 'Search...', 'tmusers' ),
			"processing" => esc_html__( 'Processing...', 'tmusers' ),
			"csvHtml5" => esc_html__( 'CSV', 'tmusers' ),
			"excelHtml5" => esc_html__( 'Excel', 'tmusers' ),
			"pdfHtml5" => esc_html__( 'PDF', 'tmusers' ),
			"print" => esc_html__( 'Print', 'tmusers' )
		);
		wp_localize_script('tmusers-table', 'tmusers_table_vars', $tmusers_table_param);
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/h-card.php' );
		require_once( __DIR__ . '/widgets/v-card.php' );
		require_once( __DIR__ . '/widgets/t-card.php' );
		require_once( __DIR__ . '/widgets/table.php' );
	}

	public function register_widgets() {
		$this->include_widgets_files();

		// Check bbPress Plugin
		if (class_exists( 'bbPress' )) {
			\Elementor\Plugin::instance()->widgets_manager->register( new Widgets\TMusers_H_Card() );
			\Elementor\Plugin::instance()->widgets_manager->register( new Widgets\TMusers_V_Card() );
			\Elementor\Plugin::instance()->widgets_manager->register( new Widgets\TMusers_T_Card() );
			\Elementor\Plugin::instance()->widgets_manager->register( new Widgets\TMusers_Table() );
		}
	}

	public function widget_category($elements_manager) {
		$elements_manager->add_category(
			'tmusers-widgets',
			[
				'title' => esc_html__( 'TMusers', 'tmusers' ),
				'icon' => 'fa fa-plug',
			]
		);
	}

	public function __construct() {
		add_action( 'elementor/frontend/after_enqueue_styles', [ $this, 'widget_styles' ] );
		add_action( 'elementor/frontend/after_register_scripts', [ $this, 'widget_scripts' ] );
		add_action( 'elementor/elements/categories_registered', [ $this, 'widget_category' ] );
		add_action( 'elementor/widgets/register', [ $this, 'register_widgets' ] );
	}
}

TMusersLoadElementor::instance();