<?php
defined( 'ABSPATH' ) || exit;

/**
* Hook in and register a metabox to handle a plugin options page and adds a menu item.
*/
function tmusers_register_main_options_metabox() {

/**
 * Registers main options page menu item and form.
 */
$args = array(
    'id'           => 'tmusers_options',
    'title'        => esc_html('TMusers', 'tmusers'),
    'object_types' => array( 'options-page' ),
    'option_key'   => 'tmusers_options',
    'tab_group'    => 'tmusers_options',
    'tab_title'    => esc_html('General', 'tmusers')
);

// 'tab_group' property is supported in > 2.4.0.
if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
    $args['display_cb'] = 'tmusers_options_display_with_tabs';
}

$main_options = new_cmb2_box( $args );

/**
 * Options fields ids only need
 * to be unique within this box.
 * Prefix is not needed.
 */

$main_options->add_field( array(
	'name' => esc_html__( 'Date Format', 'tmusers' ),
    'desc' => esc_html__( 'https://wordpress.org/support/article/formatting-date-and-time/', 'tmusers' ),
	'id'   => 'tmu_date_format',
	'type' => 'text',
    'default' => 'F j, Y',
) );

$main_options->add_field(
    array(
        'name' => esc_html__( 'Verified Users', 'tmusers'),  
        'id' => 'tmu_enable_verified_users',
        'type' => 'radio_inline',
        'options' => array(
            'enable' => esc_html__( 'Enable', 'tmusers' ),
            'disable'   => esc_html__( 'Disable', 'tmusers' )
        ),
        'default' => 'enable',
    )
);

$main_options->add_field(
    array(
        'name' => esc_html__( 'Last Login', 'tmusers'),  
        'desc' => esc_html__( 'Last login time is started to recorded after the plugin activation. If a user did not login after the plugin activation, registration date will be displayed as last login time for the user.', 'tmusers' ),
        'id' => 'tmu_enable_last_login',
        'type' => 'radio_inline',
        'options' => array(
            'enable' => esc_html__( 'Enable', 'tmusers' ),
            'disable'   => esc_html__( 'Disable', 'tmusers' )
        ),
        'default' => 'enable',
    )
);

/**
 * Registers new options page, and set main item as parent.
 */
$args = array(
    'id'           => 'tmusers_options_1',
    'title'        => esc_html('TMusers', 'tmusers'),
    'menu_title'   => esc_html('Avatars', 'tmusers'),
    'object_types' => array( 'options-page' ),
    'option_key'   => 'tmusers_options_1',
    'parent_slug'  => 'tmusers_options',
    'tab_group'    => 'tmusers_options',
    'tab_title'    => esc_html('Avatars', 'tmusers'),
);

// 'tab_group' property is supported in > 2.4.0.
if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
    $args['display_cb'] = 'tmusers_options_display_with_tabs';
}

$options_1 = new_cmb2_box( $args );

$options_1->add_field( array(
	'name' => esc_html__( 'Avatar Size (H-Card)', 'tmusers' ),
    'desc' => esc_html__( 'Default avatar size.', 'tmusers' ),
	'id'   => 'tmu_avatar_size_h_card',
	'type' => 'text',
	'attributes' => array(
		'type' => 'number',
		'pattern' => '\d*',
    ),
    'default' => 128,
) );

$options_1->add_field( array(
	'name' => esc_html__( 'Avatar Size (V-Card)', 'tmusers' ),
    'desc' => esc_html__( 'Default avatar size.', 'tmusers' ),
	'id'   => 'tmu_avatar_size_v_card',
	'type' => 'text',
	'attributes' => array(
		'type' => 'number',
		'pattern' => '\d*',
    ),
    'default' => 150,
) );

$options_1->add_field( array(
	'name' => esc_html__( 'Avatar Size (T-Card)', 'tmusers' ),
    'desc' => esc_html__( 'Default avatar size.', 'tmusers' ),
	'id'   => 'tmu_avatar_size_T_card',
	'type' => 'text',
	'attributes' => array(
		'type' => 'number',
		'pattern' => '\d*',
    ),
    'default' => 256,
) );

/**
 * Registers new options page, and set main item as parent.
 */
$args = array(
    'id'           => 'tmusers_options_2',
    'title'        => esc_html('TMusers', 'tmusers'),
    'menu_title'   => esc_html('Role Names', 'tmusers'),
    'object_types' => array( 'options-page' ),
    'option_key'   => 'tmusers_options_2',
    'parent_slug'  => 'tmusers_options',
    'tab_group'    => 'tmusers_options',
    'tab_title'    => esc_html('Role Names', 'tmusers'),
);

// 'tab_group' property is supported in > 2.4.0.
if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
    $args['display_cb'] = 'tmusers_options_display_with_tabs';
}

$options_2 = new_cmb2_box( $args );

$options_2->add_field(
    array(
        'name' => esc_html__( 'Custom Role Names', 'tmusers'),  
        'id' => 'tmu_enable_role_names',
        'type' => 'radio_inline',
        'options' => array(
            'enable' => esc_html__( 'Enable', 'tmusers' ),
            'disable'   => esc_html__( 'Disable', 'tmusers' )
        ),
        'default' => 'disable',
    )
);

$options_2->add_field( array(
	'name' => esc_html__( 'Keymaster', 'tmusers' ),
	'id'   => 'tmu_keymaster_name',
	'type' => 'text',
    'default' => esc_html__( 'Keymaster', 'tmusers' )
) );

$options_2->add_field( array(
	'name' => esc_html__( 'Moderator', 'tmusers' ),
	'id'   => 'tmu_moderator_name',
	'type' => 'text',
    'default' => esc_html__( 'Moderator', 'tmusers' )
) );

$options_2->add_field( array(
	'name' => esc_html__( 'Participant', 'tmusers' ),
	'id'   => 'tmu_participant_name',
	'type' => 'text',
    'default' => esc_html__( 'Participant', 'tmusers' )
) );

$options_2->add_field( array(
	'name' => esc_html__( 'Spectator', 'tmusers' ),
	'id'   => 'tmu_spectator_name',
	'type' => 'text',
    'default' => esc_html__( 'Spectator', 'tmusers' )
) );

$options_2->add_field( array(
	'name' => esc_html__( 'Blocked', 'tmusers' ),
	'id'   => 'tmu_blocked_name',
	'type' => 'text',
    'default' => esc_html__( 'Blocked', 'tmusers' )
) );

}
add_action( 'cmb2_admin_init', 'tmusers_register_main_options_metabox' );

/**
* A CMB2 options-page display callback override which adds tab navigation among
* CMB2 options pages which share this same display callback.
*
* @param CMB2_Options_Hookup $cmb_options The CMB2_Options_Hookup object.
*/
function tmusers_options_display_with_tabs( $cmb_options ) {
$tabs = tmusers_options_page_tabs( $cmb_options );
?>
<div class="wrap cmb2-options-page option-<?php echo esc_attr($cmb_options->option_key); ?>">
    <?php if ( get_admin_page_title() ) : ?>
        <h2><?php echo wp_kses_post( get_admin_page_title() ); ?></h2>
    <?php endif; ?>
    <h2 class="nav-tab-wrapper">
        <?php foreach ( $tabs as $option_key => $tab_title ) : ?>
            <a class="nav-tab<?php if ( isset( $_GET['page'] ) && $option_key === $_GET['page'] ) : ?> nav-tab-active<?php endif; ?>" href="<?php menu_page_url( $option_key ); ?>"><?php echo wp_kses_post( $tab_title ); ?></a>
        <?php endforeach; ?>
    </h2>
    <form class="cmb-form" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>" method="POST" id="<?php echo esc_attr($cmb_options->cmb->cmb_id); ?>" enctype="multipart/form-data" encoding="multipart/form-data">
        <input type="hidden" name="action" value="<?php echo esc_attr( $cmb_options->option_key ); ?>">
        <?php $cmb_options->options_page_metabox(); ?>
        <?php submit_button( esc_attr( $cmb_options->cmb->prop( 'save_button' ) ), 'primary', 'submit-cmb' ); ?>
    </form>
</div>
<?php
}

/**
* Gets navigation tabs array for CMB2 options pages which share the given
* display_cb param.
*
* @param CMB2_Options_Hookup $cmb_options The CMB2_Options_Hookup object.
*
* @return array Array of tab information.
*/
function tmusers_options_page_tabs( $cmb_options ) {
$tab_group = $cmb_options->cmb->prop( 'tab_group' );
$tabs      = array();

foreach ( CMB2_Boxes::get_all() as $cmb_id => $cmb ) {
    if ( $tab_group === $cmb->prop( 'tab_group' ) ) {
        $tabs[ $cmb->options_page_keys()[0] ] = $cmb->prop( 'tab_title' )
            ? $cmb->prop( 'tab_title' )
            : $cmb->prop( 'title' );
    }
}

return $tabs;
}

/**
* Get option
*/

function tmusers_get_option( $tab = 'tmusers_options', $key = '', $default = false ) {
	if ( function_exists( 'cmb2_get_option' ) ) {
		return cmb2_get_option( $tab, $key, $default );
	}

	$opts = get_option( $tab, $default );

	$val = $default;

	if ( 'all' == $key ) {
		$val = $opts;
	} elseif ( is_array( $opts ) && array_key_exists( $key, $opts ) && false !== $opts[ $key ] ) {
		$val = $opts[ $key ];
	}

	return $val;
}