<?php
defined( 'ABSPATH' ) || exit;

class TMusersVerified  
{
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action('init', array($this, 'init'));
    }

	/* Init */
    public function init() {
		add_action( 'admin_enqueue_scripts', array($this, 'add_scripts') );
        add_filter( 'manage_users_columns', array($this, 'add_columns'), 99 );
		add_action( 'manage_users_custom_column',  array($this, 'add_rows'), 10, 3 );
		add_action( 'wp_ajax_tmusers_toggle_verified_user_status', array($this, 'toggle_status') );
		add_action( 'edit_user_profile', array($this, 'add_checkbox') ,999 );
		add_action( 'edit_user_profile_update', array($this, 'update_option'));
    }

	/* Add scripts */
	public function add_scripts($hook){
		if (('users.php' == $hook) || ('user-edit.php' == $hook)) {
			$tmusers_js_script_ajax_nonce = wp_create_nonce( "tmusers_js_script_ajax_nonce" );
			wp_enqueue_style('tmusers-verified-users', plugin_dir_url( __FILE__ ) . 'css/verified-users.css', false, '1.0');
			wp_enqueue_script('tmusers-verified-users', plugin_dir_url( __FILE__ ) . 'js/verified-users.js', array( 'jquery' ), '1.0', true );
			wp_localize_script( 'tmusers-verified-users', 'tmusers_vars', array( 'tmusers_ajax_url'   => admin_url( 'admin-ajax.php' ),'tmusers_plugin_dir' => plugins_url('',__FILE__) ,'tmusers_js_script_ajax_nonce'=>$tmusers_js_script_ajax_nonce)); 
		} else {
			return;
		}
	}

	/* Add columns */
	public function add_columns( $column ) {
		$column['user_id'] =  esc_html__( 'User ID', 'tmusers');
		$column['tmusers_verified_user'] = '<img src="' . plugin_dir_url( __FILE__ ) . 'images/verified.png" title="' . esc_attr__( 'Verified', 'tmusers') . '" />';
		return apply_filters('tmusers_verified_col_name',$column);
	}

	/* Add rows */
	public function add_rows( $val, $column_name, $user_id ) {    
		$user = get_userdata( $user_id );
		if ( 'user_id' == $column_name ) {
			return $user_id;
		} else if ( 'tmusers_verified_user' == $column_name ) {
			$is_verified_user = sanitize_text_field(get_user_meta( $user_id , 'tmusers_verified_user' ,true ));
			if( $is_verified_user ) {
				return '<img src="'. apply_filters('tmusers_verified_user_img',
					plugins_url( 'images/active_user.png', __FILE__ )) .'" class="tmusers_verified_user" verified="yes" user-id="'. $user_id .'" >';
			}
			else {
				return '<img src="'. apply_filters('tmusers_not_verified_user_img',
					plugins_url( 'images/inactive_user.png', __FILE__ )).'" class="tmusers_verified_user" verified="no" user-id="'. $user_id .'" >';
			} 
		} else {
			return $val; 
		}
	}

	/* User status description */
	public function toggle_status() {
		if ( !current_user_can( 'edit_user', $user_id ) ) return false;

		check_ajax_referer( 'tmusers_js_script_ajax_nonce', 'tmusers_js_script_ajax_nonce' );

		$is_verified_user = sanitize_text_field($_POST["verified"]);
		$user_id = intval($_POST["user_id"]);

		if( $is_verified_user == 'yes' )
		{
			update_user_meta( $user_id, 'tmusers_verified_user' , 'yes' );
		} 
		else {
			delete_user_meta( $user_id, 'tmusers_verified_user' );
		}

		echo esc_attr__( 'User verified Status Is Changed', 'tmusers');
	
	}

	/* Add checkbox option to the user edit page */
	public function add_checkbox( $user ){    
		$user_id = $user->ID;               
		$is_verified_user  = sanitize_text_field(get_user_meta( $user_id, "tmusers_verified_user", true ));               
		?> 
		<table class="form-table">
			<tr class="user-admin-bar-front-wrap">
				<th scope="row"><?php esc_html_e( 'Verified User', 'tmusers'); ?></th>
				<td><fieldset><legend class="screen-reader-text"><span><?php esc_attr_e( 'Verified User', 'tmusers'); ?></span></legend>
					<label for="tmusers_verified_user">
					<input name="tmusers_verified_user" type="checkbox" id="tmusers_verified_user" value="yes" <?php if ( $is_verified_user == 'yes' ){ ?> checked="checked"<?php } ?> >
					<?php esc_attr_e( 'Verify this user', 'tmusers'); ?></label><br>
					</fieldset>
				</td>
			</tr>		
		</table>   
		<?php    
	} 

	/* Save the verified option in the user edit page */
	public function update_option( $user_id ){ 	        
		if ( !current_user_can( 'edit_user', $user_id ) ) return false;

		// update this users meta
		if ( isset( $_POST['tmusers_verified_user'] ) && sanitize_text_field($_POST['tmusers_verified_user']) == "yes" )
		{                     
			update_user_meta( $user_id, 'tmusers_verified_user' , 'yes' );
		}
		else {
			delete_user_meta( $user_id, 'tmusers_verified_user' );
		}                           
	} 
	}

/**
 * Returns the main instance of TMusersVerified.
 */

function TMusersVerified() {  
	return TMusersVerified::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMusersVerified'] = TMusersVerified();