<?php
namespace ElementorTMusers\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use WP_User_Query;
use Elementor\Group_Control_Css_Filter;

if ( ! defined( 'ABSPATH' ) ) exit;

class TMusers_H_Card extends Widget_Base {

	public function get_name() {
		return 'tmusers-h-card';
	}

	public function get_title() {
		return esc_html__( 'TMusers H-Card', 'tmusers' );
	}

	public function get_icon() {
		return 'eicon-person';
	}

	public function get_categories() {
		return [ 'tmusers-widgets' ];
	}

    public function get_style_depends() {
		return [ 'elementor-icons-fa-solid', 'tmusers-components', 'tmusers-h-card' ];
	}
    
    public function get_script_depends() {
		return [ 'tmusers-h-card' ];
	}

	protected function _register_controls() {
		$this->start_controls_section(
			'settings_section',
			[
				'label' => esc_html__( 'Settings', 'tmusers' ),
                'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

        $this->add_responsive_control(
			'columns',
			[
				'label' => esc_html__( 'Columns', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 1,
						'max' => 6,
						'step' => 1,
					],
				],
				'desktop_default' => [
					'size' => 2,
					'unit' => 'px',
				],
				'tablet_default' => [
					'size' => 2,
					'unit' => 'px',
				],
				'mobile_default' => [
					'size' => 1,
					'unit' => 'px',
				],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-grid' => 'grid-template-columns:repeat({{SIZE}}, 1fr);'
				],
			]
		);
        
        $this->add_responsive_control(
			'gap',
			[
				'label' => esc_html__( 'Grid Gap (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 20,
				],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-grid' => 'grid-gap: {{SIZE}}{{UNIT}};'
				],
			]
		);

        $this->add_control(
			'section_settings_hr',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

        $this->add_control(
			'orderby',
			[
				'label' => esc_html__( 'Order By', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'titleasc',
				'options' => [
                    'titleasc' => esc_html__( 'Alphabetical ASC', 'tmusers' ),
                    'titledesc' => esc_html__( 'Alphabetical DESC', 'tmusers' ),
                    'new' => esc_html__( 'New', 'tmusers' ),
                    'old' => esc_html__( 'Old', 'tmusers' ),
                    'topics' => esc_html__( 'Most Topics', 'tmusers' ),
                    'replies' => esc_html__( 'Most Replies', 'tmusers' )
				],
			]
		);

        $this->add_control(
			'limit',
			[
				'label' => esc_html__( 'Maximum number of users', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 99,
				'step' => 1,
				'default' => 12,
			]
		);

        $this->add_control(
			'pagination', [
				'label' => esc_html__( 'Pagination', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
				'show_label' => true
			]
		);

        $this->add_control(
			'exclude', [
				'label' => esc_html__( 'Exclude users by ID', 'tmusers' ),
                'description' => esc_html__( 'To exclude multiple users, add comma between IDs.', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => ''
			]
		);

		$this->add_control(
			'include', [
				'label' => esc_html__( 'Include users by ID', 'tmusers' ),
                'description' => esc_html__( 'To include multiple users, add comma between IDs.', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => ''
			]
		);
     
        $this->add_control(
			'verified', [
				'label' => esc_html__( 'Only Verified Users', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
				'show_label' => true,
			]
		);

        $this->end_controls_section();

        $this->start_controls_section(
			'visibility_section',
			[
				'label' => esc_html__( 'Visibility', 'tmusers' ),
                'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'show_search', [
				'label' => esc_html__( 'Search Form', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'show_label' => true,
			]
		);

		$this->add_control(
			'search_label', [
				'label' => esc_html__( 'Search Label', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Search by name', 'tmusers' ),
                'condition' => ['show_search' => 'yes'],
			]
		);

		$this->add_control(
			'orderby_label', [
				'label' => esc_html__( 'Order by Label', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Order by', 'tmusers' ),
                'condition' => ['show_search' => 'yes'],
			]
		);

        $this->add_control(
			'show_avatar', [
				'label' => esc_html__( 'Avatar', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'show_label' => true,
			]
		);

        $this->add_control(
			'show_role', [
				'label' => esc_html__( 'Role', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'show_label' => true,
			]
		);

        $this->add_control(
			'show_topics', [
				'label' => esc_html__( 'Topics Started', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'show_label' => true,
			]
		);

        $this->add_control(
			'topics_label', [
				'label' => esc_html__( 'Topics Label', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Topics', 'tmusers' ),
                'condition' => ['show_topics' => 'yes'],
			]
		);

        $this->add_control(
			'show_replies', [
				'label' => esc_html__( 'Replies Created', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'show_label' => true,
			]
		);

        $this->add_control(
			'replies_label', [
				'label' => esc_html__( 'Replies Label', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Replies', 'tmusers' ),
                'condition' => ['show_replies' => 'yes'],
			]
		);
        
        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_form_container_style',
			[
				'label' => esc_html__( 'Search Form', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => ['show_search' => 'yes']
			]
		);

		$this->add_control(
			'form_reverse', [
				'label' => esc_html__( 'Reverse Row', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
				'show_label' => true,
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'form_container_bg',
				'label' => esc_html__( 'Background', 'tmusers' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .tmusers-filters',
			]
		);

		$this->add_control(
			'form_container_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'form_container_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-filters'
			]
		);
        
        $this->add_responsive_control(
			'form_container_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmusers-filters' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'form_container_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-filters'
			]
		);

		$this->add_control(
			'form_container_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'form_container_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-filters' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'form_container_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-filters' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_search_form_style',
			[
				'label' => esc_html__( 'Form Fields', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => ['show_search' => 'yes']
			]
		);
        
        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'input_typography',		
				'selector' => '{{WRAPPER}} .tmu-form-field'
			]
		);
        
        $this->add_control(
			'input_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->start_controls_tabs( 'form_input_style' );
        
        $this->start_controls_tab(
			'form_input_normal',
			[
				'label' => esc_html__( 'Normal', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'input_text_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'input_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'input_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-form-field'
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'input_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-form-field'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'form_input_hover',
			[
				'label' => esc_html__( 'Focus', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'input_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field:focus' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'input_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field:focus' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'input_hover_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-form-field:focus'
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'input_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-form-field:focus'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'input_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'input_height',
			[
				'label' => esc_html__( 'Height (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 10,
				'max' => 100,
				'step' => 1,
                'default' => 40,
                'selectors' => [
					'{{WRAPPER}} .tmu-form-field' => 'height: {{VALUE}}px;line-height: {{VALUE}}px;',
					'{{WRAPPER}} .tmu-btn' => 'height: {{VALUE}}px;line-height: {{VALUE}}px;'
				],
			]
		);
        
        $this->add_responsive_control(
			'input_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'input_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);
        
        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_btn_style',
			[
				'label' => esc_html__( 'Search Button', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => ['show_search' => 'yes']
			]
		);
        
        $this->start_controls_tabs( 'tabs_button_style' );
        
        $this->start_controls_tab(
			'tab_button_normal',
			[
				'label' => esc_html__( 'Normal', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'btn_text_color',
			[
				'label' => esc_html__( 'Icon Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
                'default' => '#ffffff',
				'selectors' => [
					'{{WRAPPER}} .tmu-btn' => 'color: {{VALUE}};',
				]
			]
		);

		$this->add_control(
			'delete_color',
			[
				'label' => esc_html__( 'Delete Icon Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field-delete' => 'color: {{VALUE}};',
				]
			]
		);
        
        $this->add_control(
			'btn_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
				'selectors' => [
					'{{WRAPPER}} .tmu-btn' => 'background-color: {{VALUE}};',
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'btn_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-btn'
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-btn'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'tab_button_hover',
			[
				'label' => esc_html__( 'Hover', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'btn_text_hover_color',
			[
				'label' => esc_html__( 'Icon Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmu-btn:hover' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'delete_color_hover',
			[
				'label' => esc_html__( 'Delete Icon Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmu-form-field-delete:hover' => 'color: {{VALUE}};',
				]
			]
		);
        
        $this->add_control(
			'btn_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmu-btn:hover' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'btn_bg_color_hover_gradient',
				'label' => esc_html__( 'Background', 'tmusers' ),
				'types' => [ 'gradient' ],
				'selector' => '{{WRAPPER}} .tmu-btn:hover',
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'btn_hover_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-btn:hover'
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmu-btn:hover'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();

		$this->add_control(
			'btn_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'btn_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmu-btn' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'btn_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmu-btn' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_card_style',
			[
				'label' => esc_html__( 'Card', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE
			]
		);

		$this->add_control(
			'card_reverse', [
				'label' => esc_html__( 'Reverse Row', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
				'show_label' => true,
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'card_bg',
				'label' => esc_html__( 'Background', 'tmusers' ),
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .tmusers-hr-card',
			]
		);

		$this->add_control(
			'card_hr',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'card_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-card'
			]
		);
        
        $this->add_responsive_control(
			'card_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-card' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'card_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-card'
			]
		);

		$this->add_control(
			'card_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'card_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-hr-card' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_avatar_style',
			[
				'label' => esc_html__( 'Avatar', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => ['show_avatar' => 'yes']
			]
		);

		$this->add_control(
			'avatar_animation',
			[
				'label' => esc_html__( 'Hover Animation', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::HOVER_ANIMATION
			]
        );

		$this->add_group_control(
			Group_Control_Css_Filter::get_type(),
			[
				'name' => 'thumb_css_filter',
				'label' => esc_html__( 'CSS Filters', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-thumb a img'
			]
		);

        $this->add_group_control(
			Group_Control_Css_Filter::get_type(),
			[
				'name' => 'thumb_css_filter_hover',
				'label' => esc_html__( 'CSS Filters Hover', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-thumb a:hover img'
			]
		);

		$this->add_responsive_control(
			'avatar_width',
			[
				'label' => esc_html__( 'Width (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 150,
				'step' => 1,
                'default' => 64,
                'selectors' => [
					'{{WRAPPER}} .tmusers-hr-thumb' => 'width: {{VALUE}}px;min-width: {{VALUE}}px;'
				],
			]
		);

		$this->add_control(
			'avatar_hr_1',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'avatar_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-thumb'
			]
		);

		$this->add_responsive_control(
			'avatar_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-thumb' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} .tmusers-hr-thumb a' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} .tmusers-hr-thumb img' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'avatar_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-hr-thumb'
			]
		);
        
        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_card_content_style',
			[
				'label' => esc_html__( 'Name & Role', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE
			]
		);

		$this->add_control(
			'card_text_align',
			[
				'label' => esc_html__( 'Text Align', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'tmusers' ),
						'icon' => 'fas fa-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'tmusers' ),
						'icon' => 'fas fa-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'tmusers' ),
						'icon' => 'fas fa-align-right',
					]
				],
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-info' => 'text-align: {{VALUE}};'
				],
				'toggle' => true
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'name_typography',	
				'label' => esc_html__( 'Name Typography', 'tmusers' ),	
				'selector' => '{{WRAPPER}} .tmusers-hr-name a, {{WRAPPER}} .tmusers-hr-name'
			]
		);

		$this->add_control(
			'name_color',
			[
				'label' => esc_html__( 'Name Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-name' => 'color: {{VALUE}};',
					'{{WRAPPER}} .tmusers-hr-name a' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'name_hover_color',
			[
				'label' => esc_html__( 'Name Hover Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-name a:hover' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'card_content_hr_1',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'role_typography',	
				'label' => esc_html__( 'Role Typography', 'tmusers' ),	
				'selector' => '{{WRAPPER}} .tmusers-hr-desc span,{{WRAPPER}} .tmusers-hr-desc'
			]
		);

		$this->add_control(
			'role_color',
			[
				'label' => esc_html__( 'Role Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-desc' => 'color: {{VALUE}};',
					'{{WRAPPER}} .tmusers-hr-desc span' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'role_keymaster_color',
			[
				'label' => esc_html__( 'Keymaster Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-desc span.bbp_keymaster' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'role_moderator_color',
			[
				'label' => esc_html__( 'Moderator Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-desc span.bbp_moderator' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'card_content_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'card_content_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-hr-info' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_statistics_style',
			[
				'label' => esc_html__( 'Statistics', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'conditions'   => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name'  => 'show_topics',
                            'value' => 'yes',
                        ],
                        [
                            'name'  => 'show_replies',
                            'value' => 'yes',
                        ]
                    ]
                ],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'statistic_typography',	
				'label' => esc_html__( 'Typography', 'tmusers' ),	
				'selector' => '{{WRAPPER}} .tmusers-hr-number'
			]
		);

		$this->add_control(
			'statistic_color',
			[
				'label' => esc_html__( 'Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-number' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'statistic_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-topics' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .tmusers-hr-replies' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'statistic_border_color',
			[
				'label' => esc_html__( 'Border Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-topics' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} .tmusers-hr-replies' => 'border-color: {{VALUE}};'
				]
			]
		);

		$this->add_responsive_control(
			'statistic_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-topics' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} .tmusers-hr-replies' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);

		$this->add_control(
			'statistic_hr_1',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'label_typography',	
				'label' => esc_html__( 'Label Typography', 'tmusers' ),	
				'selector' => '{{WRAPPER}} .tmusers-hr-label'
			]
		);

		$this->add_control(
			'label_color',
			[
				'label' => esc_html__( 'Label Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-label' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'label_bg_color',
			[
				'label' => esc_html__( 'Label Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-label' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'label_border_color',
			[
				'label' => esc_html__( 'Label Border Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-hr-label' => 'border-color: {{VALUE}};'
				]
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_pagination_style',
			[
				'label' => esc_html__( 'Pagination', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
				'condition' => ['pagination' => 'yes']
			]
		);

		$this->add_responsive_control(
			'pagination_item_size',
			[
				'label' => esc_html__( 'Item Size (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 10,
				'max' => 100,
				'step' => 1,
                'default' => 36,
                'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item' => 'min-width: {{VALUE}}px;line-height: {{VALUE}}px;'
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'pagination_item_typography',	
				'label' => esc_html__( 'Typography', 'tmusers' ),	
				'selector' => '{{WRAPPER}} li.tmusers-page-item'
			]
		);

		$this->start_controls_tabs( 'pagination_item_tabs' );
        
        $this->start_controls_tab(
			'pagination_item_tab',
			[
				'label' => esc_html__( 'Normal', 'tmusers'),
			]
		);
        
		$this->add_control(
			'pagination_item_color',
			[
				'label' => esc_html__( 'Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item a' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'pagination_item_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'pagination_item_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} li.tmusers-page-item'
			]
		);
        
        $this->add_responsive_control(
			'pagination_item_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'pagination_item_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} li.tmusers-page-item'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'pagination_item_tab_hover',
			[
				'label' => esc_html__( 'Active & Hover', 'tmusers'),
			]
		);

		$this->add_control(
			'pagination_item_color_hover',
			[
				'label' => esc_html__( 'Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item.active' => 'color: {{VALUE}};',
					'{{WRAPPER}} li.tmusers-page-item:hover' => 'color: {{VALUE}};',
					'{{WRAPPER}} li.tmusers-page-item.active a' => 'color: {{VALUE}};',
					'{{WRAPPER}} li.tmusers-page-item:hover a' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'pagination_item_bg_color_hover',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item.active' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} li.tmusers-page-item:hover' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'pagination_item_border_hover',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} li.tmusers-page-item.active,{{WRAPPER}} li.tmusers-page-item:hover'
			]
		);
        
        $this->add_responsive_control(
			'pagination_item_border_radius_hover',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} li.tmusers-page-item.active' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
					'{{WRAPPER}} li.tmusers-page-item:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'pagination_item_shadow_hover',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} li.tmusers-page-item.active, {{WRAPPER}} li.tmusers-page-item:hover'
			]
		);

        $this->end_controls_tab();
        $this->end_controls_tabs();

		$this->add_control(
			'pagination_hr_2',
			[
				'type' => Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'pagination_h_align',
			[
				'label' => esc_html__( 'Horizontal Align', 'tmusers' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'flex-start' => [
						'title' => esc_html__( 'Start', 'tmusers' ),
						'icon' => 'eicon-h-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'tmusers' ),
						'icon' => 'eicon-h-align-center',
					],
					'flex-end' => [
						'title' => esc_html__( 'End', 'tmusers' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'default' => 'flex-end',
				'selectors' => [
					'{{WRAPPER}} ul.tmusers-pagination' => 'justify-content: {{VALUE}};'
				],
                'toggle' => false
			]
		);

		$this->add_responsive_control(
			'pagination_margin',
			[
				'label' => esc_html__( 'Container Margin', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} ul.tmusers-pagination' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'pagination_item_margin',
			[
				'label' => esc_html__( 'Item Margin', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} li.tmusers-page-item' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();

		// section start
		$this->start_controls_section(
			'section_notice_style',
			[
				'label' => esc_html__( 'Warning Message', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'notice_typography',		
				'selector' => '{{WRAPPER}} .tmusers-notice'
			]
		);

		$this->add_control(
			'notice_color',
			[
				'label' => esc_html__( 'Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-notice' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'notice_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmusers-notice' => 'background-color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'notice_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'notice_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-notice'
			]
		);
        
        $this->add_responsive_control(
			'notice_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .tmusers-notice' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'notice_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .tmusers-notice'
			]
		);

		$this->add_control(
			'notice_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

		$this->add_responsive_control(
			'notice_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-notice' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->add_responsive_control(
			'notice_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmusers-notice' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

		$this->end_controls_section();
	}
    
    protected function render() {
		global $wpdb;
		$database_prefix = $wpdb->prefix; // Required for WP multisite
		$settings = $this->get_settings_for_display();
		$search = $settings['show_search'];
		$search_label = $settings['search_label'];
		$query_var = '';
		if ($search == 'yes') {
			$query_var = get_query_var('tmu_user');
		}
		$form_reverse = $settings['form_reverse'];
		$card_reverse = $settings['card_reverse'];
		$limit = isset($settings['limit']) ? $settings['limit'] : 12;
		$exclude = $settings['exclude'];
		$include = $settings['include'];
		$pagination = $settings['pagination'];
		$verified = $settings['verified'];
		$avatar = $settings['show_avatar'];
		$topics = $settings['show_topics'];
		$topics_label = $settings['topics_label'];
		$replies = $settings['show_replies'];
		$replies_label = $settings['replies_label'];
		$role = $settings['show_role'];
		$roles = bbp_get_dynamic_roles();
		$orderby = $settings['orderby'];
		$orderby_label = $settings['orderby_label'];
		$search_query = array();
		if ($search == 'yes' && get_query_var('tmu_orderby')) {
			$orderby = get_query_var('tmu_orderby');
		}

		$verified_filter = array();
		if ($verified == 'yes') {
			$verified_filter = array(
				'meta_query' => array(
					array(
						'key' => 'tmusers_verified_user',
						'value' => 'yes'
					)
				)
			);
		}

		if ($exclude) {
			$exclude = explode( ',', $exclude );
		} else {
			$exclude = array();
		}

		if ($include) {
			$include = explode( ',', $include );
		} else {
			$include = array();
		}

		if ($pagination) {
			if ( get_query_var( 'paged' ) ) { $paged = get_query_var( 'paged' ); }
			elseif ( get_query_var( 'page' ) ) { $paged = get_query_var( 'page' ); }
			else { $paged = 1; }
			$offset = ($paged - 1) * $limit;
			if ($verified == 'yes') {
				$users = get_users(array(
					'meta_key'     => 'tmusers_verified_user',
					'meta_value'   => 'yes'
				));
			} else {
				$users = get_users();
			}
			$total_users = count($users);
			$total_pages = intval(ceil($total_users / $limit));
		} else {
			$offset = 0;
		}

		if ($query_var) {
			$search_query = array(
				'search'         => '*' . esc_attr( $query_var ) . '*',
				'search_columns' => array(
					'user_login',
					'user_nicename',
					'display_name'
				),
				'number'	 => 99,
				'offset'     => 0,
				'exclude' => $exclude,
				'include' => $include
				
			);  
		} else {
			$search_query = array(
				'number'	 => $limit,
				'offset'     => $offset,
				'exclude' => $exclude,
				'include' => $include
			); 
		}
		switch ($orderby) {
			case 'titleasc':
				$user_query = array(
					'orderby'	 => 'title',
					'order'		 => 'ASC'
				);
				break;
			case 'titledesc':
				$user_query = array(
					'orderby'	 => 'title',
					'order'		 => 'DESC'
				);
				break;
			case 'new':
				$user_query = array(
					'orderby'	 => 'user_registered',
					'order'		 => 'DESC'
				);
				break;
			case 'old':
				$user_query = array(
					'orderby'	 => 'user_registered',
					'order'		 => 'ASC'
				);
				break;
			case 'replies':	
				$key = $database_prefix . '_bbp_reply_count';
				$user_query = array(
					'meta_query' => array(
						'relation' => 'OR',
						array(
							'key' => $key,
							'compare' => 'NOT EXISTS'
						),
						array(
							'key' => $key,
							'compare' => 'EXISTS'
						)
					),
					'orderby' => array(
						'meta_value_num' => 'DESC'
					)
				);
				break;    
			case 'topics':
			default:
				$key = $database_prefix . '_bbp_topic_count';
				$user_query = array(
					'meta_query' => array(
						'relation' => 'OR',
						array(
							'key' => $key,
							'compare' => 'NOT EXISTS'
						),
						array(
							'key' => $key,
							'compare' => 'EXISTS'
						)
					),
					'orderby' => array(
						'meta_value_num' => 'DESC'
					)
				);
				break;
		}
		

		$final_query = new WP_User_Query($search_query + $user_query + $verified_filter);

		$total_query = count($final_query->get_results());
		?>

		<?php if ($search == 'yes') { ?>
		<form action="<?php the_permalink(); ?>" class="tmusers-filters <?php if ($form_reverse == 'yes') { echo 'reverse'; } ?>">
			<div class="tmusers-search">
				<input type="text" class="tmu-form-field tmu-form-search" minlength="3" placeholder="<?php echo esc_attr__($search_label); ?>" name="tmu_user" autocomplete="off" value="<?php if ($query_var) { echo esc_attr($query_var); } ?>" />
				<button type="submit" class="tmu-btn"><i class="fas fa-search"></i></button>
				<?php if ($query_var) { ?>
				<a href="#" class="tmu-form-field-delete"><i class="fas fa-times-circle"></i></a>
				<?php } ?>
			</div>
			<div class="tmusers-orderby">
				<select name="tmu_orderby" class="tmu-form-field" autocomplete="off">
					<option value="titleasc" <?php if($orderby == 'titleasc') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'Alphabetical ASC', 'tmusers' )); ?></option>
					<option value="titledesc" <?php if($orderby == 'titledesc') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'Alphabetical DESC', 'tmusers' )); ?></option>
					<option value="new" <?php if($orderby == 'new') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'New', 'tmusers' )); ?></option>
					<option value="old" <?php if($orderby == 'old') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'Old', 'tmusers' )); ?></option>
					<option value="topics" <?php if($orderby == 'topics') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'Topics Started', 'tmusers' )); ?></option>
					<option value="replies" <?php if($orderby == 'replies') { echo 'selected'; } ?>><?php echo esc_html($orderby_label . ': ' . __( 'Replies Created', 'tmusers' )); ?></option>
				</select>
			</div>
		</form>	
		<?php } ?>

		<?php if ($final_query->get_results()) { ?>
		<div class="tmusers-grid">
			<?php foreach ($final_query->get_results() as $user) { ?>
			<div class="tmusers-grid-item">
				<div class="tmusers-hr-card <?php if ($card_reverse == 'yes') { echo 'reverse'; } ?>">
					<?php if ($avatar == 'yes') { ?>
					<div class="tmusers-hr-thumb elementor-animation-<?php echo esc_attr($settings['avatar_animation']); ?>">
						<a href="<?php bbp_user_profile_url($user->ID); ?>">
						<?php
						$avatar_size = tmusers_get_option( 'tmusers_options_1', 'tmu_avatar_size_h_card', 128);
						echo get_avatar( $user->ID, $avatar_size);
						?>
						</a>
					</div>
					<?php } ?>
					<div class="tmusers-hr-info">
						<?php $user_info = get_userdata( $user->ID );
						$display_name = $user_info->display_name; ?>
						<div class="tmusers-hr-name"><a href="<?php bbp_user_profile_url($user->ID); ?>"><?php echo esc_html($display_name); ?></a></div>
						<?php if ($role == 'yes') { ?>
						<div class="tmusers-hr-desc">
							<?php
							if (isset($roles[bbp_get_user_role( $user->ID )]['name'])) {
								echo '<span class="' . esc_attr(bbp_get_user_role( $user->ID )) . '">' . esc_html(bbp_translate_user_role($roles[bbp_get_user_role( $user->ID )]['name'])) . '</span>';
							}
							?>
						</div>
						<?php } ?>
					</div>
					<?php if ($topics == 'yes' || $replies == 'yes') { ?>
					<div class="tmusers-hr-statistics">
						<?php if ($topics == 'yes') { ?>
						<div class="tmusers-hr-topics">
							<span class="tmusers-hr-number"><?php echo esc_html(bbp_get_user_topic_count_raw($user->ID)); ?></span>
							<span class="tmusers-hr-label"><?php echo esc_html($topics_label); ?></span>
						</div>
						<?php } ?>
						<?php if ($replies == 'yes') { ?>
						<div class="tmusers-hr-replies">
							<span class="tmusers-hr-number"><?php echo esc_html(bbp_get_user_reply_count_raw($user->ID)); ?></span>
							<span class="tmusers-hr-label"><?php echo esc_html($replies_label); ?></span>
						</div>
						<?php } ?>
					</div>   
					<?php } ?>
				</div>
			</div>
			<?php } ?>
		</div>


		<?php if ($pagination == 'yes' && empty($query_var) && empty($include)) {
			if ($total_users > $total_query) {
				$pages = paginate_links( [
					'base' => @add_query_arg('paged','%#%'),
					'format' => '',
					'current'      => max( 1, $paged ),
					'total'        => $total_pages,
					'type'         => 'array',
					'show_all'     => false,
					'end_size'     => 3,
					'mid_size'     => 1,
					'prev_next'    => true,
					'prev_text'    => '<i class="fas fa-angle-left"></i>',
					'next_text'    => '<i class="fas fa-angle-right"></i>',
					'add_args'     => false,
					'add_fragment' => ''
				]
			);
			$pagination_output = '<div class="tmusers-pager"><ul class="tmusers-pagination">';
			foreach ($pages as $page) {
				$pagination_output .= '<li class="tmusers-page-item' . (strpos($page, 'current') !== false ? ' active' : '') . '"> ' . str_replace('page-numbers', 'tmusers-page-link', $page) . '</li>';
			}
			$pagination_output .= '</ul></div>';
				echo wp_kses_post($pagination_output);  
			}
		}

} else { ?>
		<div class="tmusers-notice"><?php esc_html_e( 'No users found.', 'tmusers' ); ?></div>
<?php }
	}
}
?>