<?php
namespace ElementorTMusers\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use WP_User_Query;

if ( ! defined( 'ABSPATH' ) ) exit;

class TMusers_Table extends Widget_Base {

	public function get_name() {
		return 'tmusers-table';
	}

	public function get_title() {
		return esc_html__( 'TMusers Table', 'tmusers' );
	}

    public function get_icon() {
		return 'eicon-table';
    }

	public function get_categories() {
		return [ 'tmusers-widgets' ];
	}
    
    public function get_style_depends(){
		return [ 'elementor-icons-fa-solid', 'tmusers-components', 'tmusers-data-table', 'tmusers-table' ];
	}
    
    public function get_script_depends() {
		return [ 'tmusers-data-table', 'tmusers-table' ];
	}

	protected function _register_controls() {

        $this->start_controls_section(
			'data_fields_section',
			[
				'label' => esc_html__( 'Data Fields', 'tmusers' ),
                'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$repeater = new \Elementor\Repeater();

		$repeater->add_control(
			'data_label', [
				'label' => esc_html__( 'Label', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'Name', 'tmusers' ),
				'label_block' => true
			]
		);

		$repeater->add_control(
			'data_field',
			[
				'label' => esc_html__( 'Data', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'name',
				'options' => [
                    'name' => esc_html__( 'Name', 'tmusers' ),
                    'role' => esc_html__( 'Forum Role', 'tmusers' ),
                    'topics' => esc_html__( 'Topics Started', 'tmusers' ),
                    'replies' => esc_html__( 'Replies Created', 'tmusers' ),
                    'user_registered' => esc_html__( 'Registration Date', 'tmusers' ),
					'user_url'  => esc_html__( 'Website URL', 'tmusers' ),
                    'custom' => esc_html__( 'Custom Meta Key', 'tmusers' )
				],
				'label_block' => true
			]
		);

        $repeater->add_control(
			'meta_key',
			[
				'label' => esc_html__( 'Meta Key', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'condition' => ['data_field' => 'custom'],
				'default' => 'last_login',
				'label_block' => true
			]
		);

        $repeater->add_control(
			'table_header_style_column',
			[
				'label' => esc_html__( 'Style This Column', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
			]
        );
        
        $repeater->add_responsive_control(
			'table_header_style_column_min_width',
			[
				'label' => esc_html__( 'Minumum Width', 'tmusers' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'vw', 'rem' ],
				'range' => [
					'vw' => [
						'min' => 0,
						'max' => 100,
					],
                    'px' => [
						'min' => 0,
						'max' => 1000,
					],
                    'rem' => [
						'min' => 0,
						'max' => 100,
					],
                ],
                'condition' => ['table_header_style_column' => 'yes'],
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}}' => 'min-width: {{SIZE}}{{UNIT}};'
				]
			]
        );

		$repeater->add_control(
			'table_header_style_column_weight',
			[
				'label' => esc_html__( 'Font Weight', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'normal',
				'options' => [
					'lighter' => esc_html__( 'Lighter', 'tmusers' ),
                    'normal' => esc_html__( 'Normal', 'tmusers' ),
                    'bold' => esc_html__( 'Bold', 'tmusers' ),
                    'bolder' => esc_html__( 'Bolder', 'tmusers' )
				],
				'condition' => ['table_header_style_column' => 'yes'],
				'selectors' => [
					'{{WRAPPER}} td{{CURRENT_ITEM}}' => 'font-weight: {{VALUE}} !important;',
				]
			]
		);
        
        $repeater->add_control(
			'table_header_style_column_color',
			[
				'label' => esc_html__( 'Font Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'condition' => ['table_header_style_column' => 'yes'],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}}' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} {{CURRENT_ITEM}} a' => 'color: {{VALUE}} !important;'
				]
			]
		);
        
        $repeater->add_control(
			'table_header_style_column_bg',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'condition' => ['table_header_style_column' => 'yes'],
				'selectors' => [
					'{{WRAPPER}} {{CURRENT_ITEM}}' => 'background-color: {{VALUE}} !important;'
				]
			]
		);

		$this->add_control(
			'data_fields',
			[
				'label' => esc_html__( 'Fields', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
                'show_label' => false,
				'prevent_empty' => true,
				'default' => [
					[
                        'data_label' => esc_html__( 'Name', 'tmusers' ),
						'data_field' => 'name',
						'meta_key' => 'last_login'
					],
					[
                        'data_label' => esc_html__( 'Role', 'tmusers' ),
						'data_field' => 'role',
						'meta_key' => 'last_login'
                    ],
                    [
                        'data_label' => esc_html__( 'Topics', 'tmusers' ),
						'data_field' => 'topics',
						'meta_key' => 'last_login'
                    ],
                    [
                        'data_label' => esc_html__( 'Replies', 'tmusers' ),
						'data_field' => 'replies',
						'meta_key' => 'last_login'
                    ],
                    [
                        'data_label' => esc_html__( 'Registration Date', 'tmusers' ),
						'data_field' => 'user_registered',
						'meta_key' => 'last_login'
                    ],
				],
				'title_field' => '{{{ data_label }}}'
			]
		);

        $this->end_controls_section();

        $this->start_controls_section(
			'table_settings_section',
			[
				'label' => esc_html__( 'Settings', 'tmusers' ),
                'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
        );

        $this->add_control(
			'limit',
			[
				'label' => esc_html__( 'Maximum number of users', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 99999,
				'step' => 1,
				'default' => 99,
			]
		);

        $this->add_control(
			'exclude', [
				'label' => esc_html__( 'Exclude users by ID', 'tmusers' ),
                'description' => esc_html__( 'To exclude multiple users, add comma between IDs.', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => ''
			]
		);

		$this->add_control(
			'include', [
				'label' => esc_html__( 'Include users by ID', 'tmusers' ),
                'description' => esc_html__( 'To include multiple users, add comma between IDs.', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => ''
			]
		);
     
        $this->add_control(
			'verified', [
				'label' => esc_html__( 'Only Verified Users', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
				'default' => '',
				'show_label' => true,
			]
		);
        
        $this->add_control(
			'table_paging',
			[
				'label' => esc_html__( 'Paging', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
                'default' => 'yes'
			]
        );

        $this->add_control(
            'table_page_length',
            [
                'label' => esc_html__( 'Page Length', 'tmusers' ),
                'type' => \Elementor\Controls_Manager::NUMBER,
                'default' => 20
            ]
        );

        $this->add_control(
			'table_info',
			[
				'label' => esc_html__( 'Info', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
                'default' => 'yes'
			]
        );

        $this->add_control(
			'table_searching',
			[
				'label' => esc_html__( 'Searching', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
                'default' => 'yes'
			]
        );

        $this->add_control(
			'table_ordering',
			[
				'label' => esc_html__( 'Ordering', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
                'default' => 'yes'
			]
        );

        $this->add_control(
			'table_buttons',
			[
				'label' => esc_html__( 'Buttons', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'tmusers' ),
				'label_off' => esc_html__( 'No', 'tmusers' ),
				'return_value' => 'yes',
                'default' => 'yes'
			]
        );

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_style_section',
			[
				'label' => esc_html__( 'Table', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_responsive_control(
			'table_spacing',
			[
				'label' => esc_html__( 'Inner Spacing (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 100,
				'step' => 1,
                'default' => 10,
                'selectors' => [
					'{{WRAPPER}} table.dataTable thead th' => 'padding: {{VALUE}}px;',
					'{{WRAPPER}} table.dataTable.cell-border tbody td' => 'padding: {{VALUE}}px;'
				]
			]
		);

		$this->add_responsive_control(
			'table_top_bottom_spacing',
			[
				'label' => esc_html__( 'Top-Bottom Spacing (px)', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 100,
				'step' => 1,
                'default' => 15,
                'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_scroll' => 'padding: {{VALUE}}px 0;'
				]
			]
		);
        
        $this->add_control(
			'table_border_color',
			[
				'label' => esc_html__( 'Border Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#dddddd',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable.cell-border tbody th' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} table.dataTable.cell-border tbody td' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} table.dataTable thead th' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} table.dataTable thead td' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper.no-footer .dataTables_scrollBody' => 'border-color: {{VALUE}};'
				]
			]
        );
        
        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_header_style_section',
			[
				'label' => esc_html__( 'Table Header', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_header_typography',
				
				'selector' => '{{WRAPPER}} table.dataTable thead th',
			]
        );
        
        $this->add_control(
			'table_header_text_align',
			[
				'label' => esc_html__( 'Text Align', 'tmusers' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'tmusers' ),
						'icon' => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'tmusers' ),
						'icon' => 'eicon-text-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'tmusers' ),
						'icon' => 'eicon-text-align-right',
					],
				],
				'default' => 'left',
                'toggle' => false,
                'selectors' => [
					'{{WRAPPER}} table.dataTable thead th' => 'text-align: {{VALUE}};',
				]
			]
        );
        
        $this->add_control(
			'table_header_sorting_align',
			[
				'label' => esc_html__( 'Sorting Icon Align', 'tmusers' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'tmusers' ),
						'icon' => 'eicon-h-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'tmusers' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'default' => 'right',
				'selectors' => [
                    '{{WRAPPER}} .dataTables_scrollHead table.dataTable thead .sorting:after' => '{{VALUE}}: {{table_spacing.VALUE}}px;',
                    '{{WRAPPER}} .dataTables_scrollHead table.dataTable thead .sorting_asc:after' => '{{VALUE}}: {{table_spacing.VALUE}}px;',
                    '{{WRAPPER}} .dataTables_scrollHead table.dataTable thead .sorting_desc:after' => '{{VALUE}}: {{table_spacing.VALUE}}px;',
                    '{{WRAPPER}} .dataTables_scrollHead table.dataTable thead .sorting_asc_disabled:after' => '{{VALUE}}: {{table_spacing.VALUE}}px;',
                    '{{WRAPPER}} .dataTables_scrollHead table.dataTable thead .sorting_desc_disabled:after' => '{{VALUE}}: {{table_spacing.VALUE}}px;',
				],
                'toggle' => false
			]
		);

        $this->add_control(
			'table_header_font_color',
			[
				'label' => esc_html__( 'Font Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#252525',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable thead th' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_header_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable thead th' => 'background-color: {{VALUE}};'
				]
			]
		);
		
		$this->add_control(
			'table_header_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

        $this->add_responsive_control(
			'table_header_min_width',
			[
				'label' => esc_html__( 'Minumum Width', 'tmusers' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'vw', 'rem' ],
				'range' => [
					'vw' => [
						'min' => 0,
						'max' => 100,
					],
                    'px' => [
						'min' => 0,
						'max' => 1000,
					],
                    'rem' => [
						'min' => 0,
						'max' => 100,
					],
                ],
                'selectors' => [
                    '{{WRAPPER}} table.dataTable thead th' => 'min-width: {{SIZE}}{{UNIT}};'
				]
			]
        );

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_body_style_section',
			[
				'label' => esc_html__( 'Table Body', 'tmusers' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_body_typography',
				'selector' => '{{WRAPPER}} table.dataTable tbody tr',
			]
        );

        $this->add_control(
			'table_body_text_align',
			[
				'label' => esc_html__( 'Text Align', 'tmusers' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'tmusers' ),
						'icon' => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'tmusers' ),
						'icon' => 'eicon-text-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'tmusers' ),
						'icon' => 'eicon-text-align-right',
					],
				],
				'default' => 'left',
                'toggle' => false,
                'selectors' => [
					'{{WRAPPER}} table.dataTable tbody tr' => 'text-align: {{VALUE}};',
				]
			]
		);

        $this->add_control(
			'table_body_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

        $this->start_controls_tabs( 'table_body_styles' );
        
        $this->start_controls_tab(
			'table_body_odd',
			[
				'label' => esc_html__( 'Odd', 'tmusers'),
			]
		);
        
        $this->add_control(
			'table_body_font_color_odd',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#757575',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_link_color_odd',
			[
				'label' => esc_html__( 'Link Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd a' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_link_hover_color_odd',
			[
				'label' => esc_html__( 'Link Hover Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd a:hover' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_odd_bg',
			[
				'label' => esc_html__( 'Row Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#f9f9f9',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} table.dataTable tbody tr.odd td' => 'background-color: {{VALUE}};'
				]
			]
        );
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'table_body_even',
			[
				'label' => esc_html__( 'Even', 'tmusers'),
			]
		);

        $this->add_control(
			'table_body_font_color_even',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#757575',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_link_color_even',
			[
				'label' => esc_html__( 'Link Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even a' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_link_hover_color_even',
			[
				'label' => esc_html__( 'Link Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#000000',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even a:hover' => 'color: {{VALUE}};'
				]
			]
        );

        $this->add_control(
			'table_body_even_bg',
			[
				'label' => esc_html__( 'Row Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} table.dataTable tbody tr.even td' => 'background-color: {{VALUE}};'
				]
			]
        );

        $this->end_controls_tab();
		$this->end_controls_tabs();

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_buttons_style_section',
			[
				'label' => esc_html__( 'Buttons', 'tmusers' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'table_buttons' => 'yes'
                ],
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_btn_typography',
				
				'selector' => '{{WRAPPER}} button.dt-button,{{WRAPPER}} div.dt-button,{{WRAPPER}} a.dt-button',
			]
		);
        
        $this->add_control(
			'table_btn_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);  
        
        $this->start_controls_tabs( 'tabs_table_button_style' );
        
        $this->start_controls_tab(
			'tab_table_button_normal',
			[
				'label' => esc_html__( 'Normal', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_btn_text_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} button.dt-button' => 'color: {{VALUE}};',
                    '{{WRAPPER}} div.dt-button' => 'color: {{VALUE}};',
                    '{{WRAPPER}} a.dt-button' => 'color: {{VALUE}};',
				]
			]
		);
        
        $this->add_control(
			'table_btn_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} button.dt-button' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} div.dt-button' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} a.dt-button' => 'background-color: {{VALUE}};',
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_btn_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} button.dt-button,{{WRAPPER}} div.dt-button,{{WRAPPER}} a.dt-button'
			]
		);
        
        $this->add_responsive_control(
			'table_btn_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} button.dt-button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} div.dt-button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} a.dt-button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_btn_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} button.dt-button,{{WRAPPER}} div.dt-button,{{WRAPPER}} a.dt-button'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'tab_table_button_hover',
			[
				'label' => esc_html__( 'Hover', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_btn_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} button.dt-button:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} div.dt-button:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} a.dt-button:hover' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_btn_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} button.dt-button:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} div.dt-button:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} a.dt-button:hover' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_btn_hover_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} button.dt-button:hover,{{WRAPPER}} div.dt-button:hover,{{WRAPPER}} a.dt-button:hover'
			]
		);
        
        $this->add_responsive_control(
			'table_btn_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} button.dt-button:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} div.dt-button:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} a.dt-button:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_btn_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} button.dt-button:hover,{{WRAPPER}} div.dt-button:hover,{{WRAPPER}} a.dt-button:hover'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'table_btn_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->add_responsive_control(
			'table_btn_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} button.dt-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} div.dt-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} a.dt-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
        
        $this->add_responsive_control(
			'table_btn_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} button.dt-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} div.dt-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} a.dt-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

        $this->end_controls_section();

        // section start
		$this->start_controls_section(
			'table_pagination_style_section',
			[
				'label' => esc_html__( 'Pagination', 'tmusers' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'table_paging' => 'yes'
                ],
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_pagination_typography',
				
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button',
			]
		);
        
        $this->add_control(
			'table_pagination_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);  
        
        $this->start_controls_tabs( 'tabs_table_pagination_style' );
        
        $this->start_controls_tab(
			'tab_table_pagination_normal',
			[
				'label' => esc_html__( 'Normal', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_pagination_text_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_pagination_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_pagination_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button'
			]
		);
        
        $this->add_responsive_control(
			'table_pagination_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_pagination_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'tab_table_pagination_hover',
			[
				'label' => esc_html__( 'Hover', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_pagination_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_pagination_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_pagination_hover_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active'
			]
		);
        
        $this->add_responsive_control(
			'table_pagination_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_pagination_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'table_pagination_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->add_responsive_control(
			'table_pagination_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);
        
        $this->add_responsive_control(
			'table_pagination_margin',
			[
				'label' => esc_html__( 'Margin', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
		);

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_search_style_section',
			[
				'label' => esc_html__( 'Search Box', 'tmusers' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'table_searching' => 'yes'
                ],
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_search_typography',
				
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->add_control(
			'table_search_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->start_controls_tabs( 'table_search_style' );
        
        $this->start_controls_tab(
			'table_search_normal',
			[
				'label' => esc_html__( 'Normal', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_search_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_placeholder_color',
			[
				'label' => esc_html__( 'Placeholder Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input::placeholder' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_search_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->add_responsive_control(
			'table_search_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_search_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'table_search_hover',
			[
				'label' => esc_html__( 'Focus', 'tmusers' ),
			]
		);
        
        $this->add_control(
			'table_search_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_search_hover_border',
				'label' => esc_html__( 'Border', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus'
			]
		);
        
        $this->add_responsive_control(
			'table_search_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmusers' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_search_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmusers' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'input_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'table_search_padding',
			[
				'label' => esc_html__( 'Padding', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );
        
        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_info_style_section',
			[
				'label' => esc_html__( 'Info', 'tmusers' ),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'table_info' => 'yes'
                ],
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_info_typography',
				
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_info',
			]
        );

        $this->add_control(
			'table_info_color',
			[
				'label' => esc_html__( 'Text Color', 'tmusers' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_info' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->end_controls_section();
	}
    
    protected function render() {
        $settings = $this->get_settings_for_display(); 
        $widget_id = $this->get_id();
		$limit = isset($settings['limit']) ? $settings['limit'] : 12;
		$exclude = $settings['exclude'];
		$include = $settings['include'];
		$verified = $settings['verified'];
		$roles = bbp_get_dynamic_roles();
        $paging = $settings['table_paging'] ? 'true' : 'false';
        $pagelength = $settings['table_page_length'];
        $info = $settings['table_info'] ? 'true' : 'false';
        $searching = $settings['table_searching'] ? 'true' : 'false';
        $ordering = $settings['table_ordering'] ? 'true' : 'false';
        $buttons = $settings['table_buttons'] ? 'true' : 'false';

		$verified_filter = array();
		if ($verified == 'yes') {
			$verified_filter = array(
				'meta_query' => array(
					array(
						'key' => 'tmusers_verified_user',
						'value' => 'yes'
					)
				)
			);
		}

		if ($exclude) {
			$exclude = explode( ',', $exclude );
		} else {
			$exclude = array();
		}

		if ($include) {
			$include = explode( ',', $include );
		} else {
			$include = array();
		}

		$user_query = array(
            'orderby'	 => 'title',
			'order'		 => 'ASC',
            'number'	 => $limit,
            'offset'     => 0,
            'exclude' => $exclude,
            'include' => $include
        );

		$final_query = new WP_User_Query($user_query + $verified_filter);

		$total_query = count($final_query->get_results());
        ?>
        <?php if ($settings['data_fields']) { ?>
        <div class="tmusers-table-container" data-paging="<?php echo esc_attr($paging); ?>" data-pagelength="<?php echo esc_attr($pagelength); ?>" data-info="<?php echo esc_attr($info); ?>" data-searching="<?php echo esc_attr($searching); ?>" data-ordering="<?php echo esc_attr($ordering); ?>" data-buttons="<?php echo esc_attr($buttons); ?>">
            <table id="tmusers-table-<?php echo $widget_id; ?>" class="tmusers-table cell-border" style="width:100%;visibility:hidden;">
                <thead>
                    <tr>
                        <?php 
                        foreach ( $settings['data_fields'] as $item ) { 
                            echo '<th class="elementor-repeater-item-' . $item['_id'] . '">' . $item['data_label'] . '</th>';
                        } ?>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($final_query->get_results() as $user) { ?>
                        <tr>
                        <?php 
                        foreach ( $settings['data_fields'] as $item ) {
                            if ($item['data_field'] == 'name') {
                                $user_info = get_userdata( $user->ID );
                                $display_name = $user_info->display_name; 
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '"><a href="' . esc_url(bbp_get_user_profile_url($user->ID)) . '">' . esc_html($display_name) . '</a></td>';
                            } else if ($item['data_field'] == 'role') {
                                if (isset($roles[bbp_get_user_role( $user->ID )]['name'])) {
                                    echo '<td class="elementor-repeater-item-' . $item['_id'] . '" data-order="' . esc_attr(bbp_translate_user_role($roles[bbp_get_user_role( $user->ID )]['name'])) . '"><span class="' . esc_attr(bbp_get_user_role( $user->ID )) . '">' . esc_html(bbp_translate_user_role($roles[bbp_get_user_role( $user->ID )]['name'])) . '</span></td>';
                                } else {
                                    echo '<td class="elementor-repeater-item-' . $item['_id'] . '" data-order="zzz"></td>';
                                }
                            } else if ($item['data_field'] == 'topics') {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '">' . esc_html(bbp_get_user_topic_count_raw($user->ID)) . '</td>';
                            } else if ($item['data_field'] == 'replies') {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '">' . esc_html(bbp_get_user_reply_count_raw($user->ID)) . '</td>';
                            } else if ($item['data_field'] == 'user_registered') {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '" data-sort="' . esc_attr(strtotime($user->user_registered)) . '">' . esc_html(date( tmusers_get_option( 'tmusers_options', 'tmu_date_format', 'F j, Y'), strtotime( $user->user_registered ))) . '</td>';
                            } else if ($item['data_field'] == 'user_url') {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '">' . esc_html(str_replace(array( 'http://', 'https://' ), '', $user->user_url)) . '</td>';
                            } else if (function_exists('tmusers_get_last_login') && $item['meta_key'] == 'last_login') {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '" data-sort="' . esc_attr(tmusers_get_last_login_timestamp($user)) . '">' . esc_html(tmusers_get_last_login($user)) . '</td>';
                            } else if (!empty($item['meta_key'])) {
                                echo '<td class="elementor-repeater-item-' . $item['_id'] . '">' . esc_html(get_user_meta($user->ID, $item['meta_key'], true)) . '</td>';
                            }
                        }
                        ?>
                        </tr>
                    <?php } ?>
                </tbody>
            </table>   
        </div>
            <?php
        }
	}
}